//+------------------------------------------------------------------+
//|                                          optimal tracking filter |
//|                                                           mladen |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"

#property indicator_chart_window
#property indicator_buffers 1
#property indicator_color1  clrYellow
#property strict

//
//
//

extern ENUM_TIMEFRAMES    TimeFrame   = PERIOD_CURRENT;    // Time frame
input ENUM_APPLIED_PRICE  Price       = PRICE_MEDIAN;      // Price
input bool                Interpolate = true;              // Interpolate in multi time frame mode

double otf[],count[],buff[][3];
string indicatorFileName;
#define _mtfCall(_buff,_ind) iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,Price,_buff,_ind)

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+

int OnInit()
{
   IndicatorBuffers(2);
   SetIndexBuffer(0,otf,INDICATOR_DATA);
   SetIndexBuffer(1,count);
   
   indicatorFileName = WindowExpertName();
   TimeFrame         = fmax(TimeFrame,_Period); 
   
   IndicatorSetString(INDICATOR_SHORTNAME,timeFrameToString(TimeFrame)+" Optimal tracking filter");
return(INIT_SUCCEEDED);
}
void OnDeinit(const int reason) { }

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+

#define _prc  0
#define _tmp1 1
#define _tmp2 2

int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime& time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
{
   int i,r,limit=fmin(rates_total-prev_calculated+1,rates_total-1); count[0]=limit;
   if (ArrayRange(buff,0) != Bars) ArrayResize(buff,Bars);
      if (TimeFrame!=_Period)
      {
         limit = (int)fmax(limit,fmin(rates_total-1,_mtfCall(2,0)*TimeFrame/_Period));
         for (i=limit;i>=0 && !_StopFlag; i--)
         {
            int y = iBarShift(NULL,TimeFrame,time[i]);
               otf[i] = _mtfCall(0,y);
               
               //
               //
               //
      
               if (!Interpolate || (i>0 && y==iBarShift(NULL,TimeFrame,time[i-1]))) continue;
               #define _interpolate(buff) buff[i+k] = buff[i]+(buff[i+n]-buff[i])*k/n
               int n,k; datetime btime = iTime(NULL,TimeFrame,y);
                  for(n = 1; (i+n)<rates_total && time[i+n] >= btime; n++) continue;	
                  for(k = 1; k<n && (i+n)<rates_total && (i+k)<rates_total; k++) _interpolate(otf);                                           
           }   
	return(rates_total);
	}      
                            
   //
   //
   //
   
   for(i=limit, r=rates_total-i-1; i>=0; i--,r++)
   {
      buff[r][_prc] = iMA(NULL,0,1,0,MODE_SMA,Price,i);
      if (r>0)
      {
         buff[r][_tmp1] = 0.2*(buff[r][_prc]-buff[r-1][_prc])+0.8*buff[r-1][_tmp1];
         buff[r][_tmp2] = 0.1*(high[i]-low[i])               +0.8*buff[r-1][_tmp2];
         double lambda = (buff[r][_tmp2]!=0) ? fabs(buff[r][_tmp1]/buff[r][_tmp2]) : 0;
         double alpha = (-lambda*lambda + sqrt(lambda*lambda*lambda*lambda + 16.0*lambda*lambda))/8.0;
         otf[i] = otf[i+1]+alpha*(buff[r][_prc]-otf[i+1]);
       }
   }
return(rates_total);
}

//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}